import { GroupChannel } from '@sendbird/chat/groupChannel';
import { LoggerInterface } from '../../../../lib/Logger';
import { BaseMessage } from '@sendbird/chat/message';
/**
 * POTENTIAL IMPROVEMENT NEEDED:
 * Current implementation might have race condition issues when the hook is called multiple times in rapid succession:
 *
 * 1. Race Condition Risk:
 *    - Multiple rapid clicks on reaction buttons could trigger concurrent API calls
 *    - The server responses might arrive in different order than the requests were sent
 *    - This could lead to inconsistent UI states where the final reaction state doesn't match user's last action
 *
 * 2. Performance Impact:
 *    - Each click generates a separate API call without debouncing/throttling
 *    - Under high-frequency clicks, this could cause unnecessary server load
 *
 * But we won't address these issues for now since it's being used only in the legacy codebase.
 * */
export default function useToggleReactionCallback(currentChannel: GroupChannel | null, logger?: LoggerInterface): (message: BaseMessage, key: string, isReacted: boolean) => void;
